@@ -24,8 +24,9 @@ from collections.abc import MutableMapping, Mapping
 import email
 import email.policy
 from http.client import (HTTPS_PORT, HTTP_PORT, NO_CONTENT, NOT_MODIFIED)
-from select import EPOLLIN, EPOLLOUT
+from select import POLLIN, POLLOUT
 import select
+import sys
 
 try:
     import asyncio
@@ -33,7 +34,7 @@ except ImportError:
     asyncio = None
 
     
-__version__ = '2.2'
+__version__ = '3.0'
 
 log = logging.getLogger(__name__)
 
@@ -83,9 +84,9 @@ class PollNeeded(tuple):
         ready for IO of the specified type, operation will continue without
         blocking.
 
-        The type of IO is specified as a :ref:`epoll <epoll-objects>` compatible
-        event mask, i.e. a bitwise combination of `!select.EPOLLIN` and
-        `!select.EPOLLOUT`.
+        The type of IO is specified as a :ref:`poll <poll-objects>`
+        compatible event mask, i.e. a bitwise combination of `!select.POLLIN`
+        and `!select.POLLOUT`.
         '''
         
         return self[1]
@@ -101,10 +102,7 @@ class PollNeeded(tuple):
         '''
 
         poll = select.poll()
-        if self.mask & EPOLLIN:
-            poll.register(self.fd, select.POLLIN)
-        if self.mask & EPOLLOUT:
-            poll.register(self.fd, select.POLLOUT)
+        poll.register(self.fd, self.mask)
             
         log.debug('calling poll')
         if timeout:
@@ -564,7 +562,7 @@ class HTTPConnection:
                     raise BlockingIOError()
             except (socket.timeout, ssl.SSLWantWriteError, BlockingIOError):
                 log.debug('yielding')
-                yield PollNeeded(fd, EPOLLOUT)
+                yield PollNeeded(fd, POLLOUT)
                 continue
             except BrokenPipeError:
                 raise ConnectionClosed('found closed when trying to write')
@@ -814,6 +812,7 @@ class HTTPConnection:
 
         # Some modules like TextIOWrapper unfortunately rely on read()
         # to return bytes, and do not accept bytearrays or memoryviews.
+        # cf. http://bugs.python.org/issue21057
         if not isinstance(buf, bytes):
             buf = bytes(buf)
         return buf
@@ -843,6 +842,42 @@ class HTTPConnection:
             raise self._encoding
         else:
             raise RuntimeError('ooops, this should not be possible')
+
+    def read_raw(self, size):
+        '''Read *size* bytes of uninterpreted data
+
+        This method may be used even after `UnsupportedResponse` or
+        `InvalidResponse` has been raised. It reads raw data from the socket
+        without attempting to interpret it. This is probably only useful for
+        debugging purposes to take a look at the raw data received from the
+        server. This method blocks if no data is available, and returns ``b''``
+        if the connection has been closed.
+
+        Calling this method will break the internal state and switch the socket
+        to blocking operation. The connection has to be closed and reestablished
+        afterwards.
+
+        **Don't use this method unless you know exactly what you are doing**.
+        '''
+
+        self._sock.setblocking(True)
+        
+        buf = bytearray()
+        rbuf = self._rbuf
+        while len(buf) < size:
+            len_ = min(size - len(buf), len(rbuf))
+            if len_ < len(rbuf):
+                buf += rbuf.d[rbuf.b:rbuf.b+len_]
+                rbuf.b += len_
+            elif len_ == 0:
+                buf2 = self._sock.recv(size - len(buf))
+                if not buf2:
+                    break
+                buf += buf2
+            else:
+                buf += rbuf.exhaust()
+
+        return buf
         
     def readinto(self, buf):
         '''placeholder, will be replaced dynamically'''
@@ -901,7 +936,7 @@ class HTTPConnection:
             got_data = self._try_fill_buffer()
             if not got_data and not rbuf:
                 log.debug('buffer empty and nothing to read, yielding..')
-                yield PollNeeded(sock_fd, EPOLLIN)
+                yield PollNeeded(sock_fd, POLLIN)
             elif not got_data:
                 log.debug('nothing more to read')
                 break
@@ -964,7 +999,7 @@ class HTTPConnection:
                     return pos
                 else:
                     log.debug('no data yet and nothing to read, yielding..')
-                    yield PollNeeded(sock_fd, EPOLLIN)
+                    yield PollNeeded(sock_fd, POLLIN)
                     continue
 
             if not read:
@@ -1078,7 +1113,7 @@ class HTTPConnection:
             # Refill buffer
             while not self._try_fill_buffer():
                 log.debug('need more data, yielding')
-                yield PollNeeded(sock_fd, EPOLLIN)
+                yield PollNeeded(sock_fd, POLLIN)
 
         log.debug('found substr at %d', idx)
         idx += len(substr)
@@ -1133,7 +1168,7 @@ class HTTPConnection:
             if len(rbuf.d) - rbuf.b < len_:
                 self._rbuf.compact()
             if not self._try_fill_buffer():
-                yield PollNeeded(sock_fd, EPOLLIN)
+                yield PollNeeded(sock_fd, POLLIN)
 
     def readall(self):
         '''placeholder, will be replaced dynamically'''
@@ -1213,25 +1248,29 @@ def _extend_HTTPConnection_docstrings():
 
 _extend_HTTPConnection_docstrings()
 
-def _join(parts):
-    '''Join a sequence of byte-like objects
+if sys.version_info < (3,4,0):
+    def _join(parts):
+        '''Join a sequence of byte-like objects
 
-    This method is necessary because `bytes.join` does not work with
-    memoryviews.
-    '''
+        This method is necessary because `bytes.join` does not work with
+        memoryviews prior to Python 3.4.
+        '''
 
-    size = 0
-    for part in parts:
-        size += len(parts)
+        size = 0
+        for part in parts:
+            size += len(parts)
 
-    buf = bytearray(size)
-    i = 0
-    for part in parts:
-        len_ = len(part)
-        buf[i:i+len_] = part
-        i += len_
+        buf = bytearray(size)
+        i = 0
+        for part in parts:
+            len_ = len(part)
+            buf[i:i+len_] = part
+            i += len_
 
-    return buf
+        return buf
+else:
+    def _join(parts):
+        return b''.join(parts)
         
 def eval_coroutine(crt):
     '''Evaluate *crt* (polling as needed) and return its result'''
@@ -1387,10 +1426,10 @@ if asyncio:
                     # reader or writer? However, in practice this should not be
                     # the case: they would read or write unpredictable parts of
                     # the input/output.
-                    if io_req.mask & EPOLLIN:
+                    if io_req.mask & POLLIN:
                         self._loop.remove_reader(io_req.fd)
                         del self._read_fds[io_req.fd]
-                    if io_req.mask & EPOLLOUT:
+                    if io_req.mask & POLLOUT:
                         self._loop.remove_writer(io_req.fd)
                         del self._write_fds[io_req.fd]
                     self._io_req = None
@@ -1402,7 +1441,7 @@ if asyncio:
                                                'PollNeeded instance!'))
                 return
 
-            if io_req.mask & EPOLLIN:
+            if io_req.mask & POLLIN:
                 reader = self._read_fds.get(io_req.fd, None)
                 if reader is None:
                     log.debug('got poll needed, registering reader')
@@ -1415,7 +1454,7 @@ if asyncio:
                                          RuntimeError('There is already a read callback for this socket'))
                     return
                 
-            if io_req.mask & EPOLLOUT:
+            if io_req.mask & POLLOUT:
                 writer = self._read_fds.get(io_req.fd, None)
                 if writer is None:
                     log.debug('got poll needed, registering writer')
